@@ -74,18 +74,19 @@ static void splitLocale(const QString &aLocale,
         locale.truncate(f);
     }
 
-    f = locale.indexOf(QLatin1Char('.'));
-    if (f >= 0) {
-        charset = locale.mid(f + 1);
-        locale.truncate(f);
-    }
-
+    // now decompose into [language[_territory][.codeset][@modifier]]
     f = locale.indexOf(QLatin1Char('@'));
     if (f >= 0) {
         modifier = locale.mid(f + 1);
         locale.truncate(f);
     }
 
+    f = locale.indexOf(QLatin1Char('.'));
+    if (f >= 0) {
+        charset = locale.mid(f + 1);
+        locale.truncate(f);
+    }
+
     f = locale.indexOf(QLatin1Char('_'));
     if (f >= 0) {
         country = locale.mid(f + 1);
@@ -95,6 +96,34 @@ static void splitLocale(const QString &aLocale,
     language = locale;
 }
 
+static void appendLocaleString(QStringList &languages, const QString &value)
+{
+    // Process the value to create possible combinations.
+    QString language, country, modifier, charset;
+    splitLocale(value, language, country, modifier, charset);
+
+    if (language.isEmpty()) {
+        return;
+    }
+
+    if (!country.isEmpty() && !modifier.isEmpty()) {
+        languages +=   language + QLatin1Char('_')
+                       + country + QLatin1Char('@')
+                       + modifier;
+    }
+    // NOTE: Priority is unclear in case both the country and
+    // the modifier are present. Should really language@modifier be of
+    // higher priority than language_country?
+    // In at least one case (Serbian language), it is better this way.
+    if (!modifier.isEmpty()) {
+        languages += language + QLatin1Char('@') + modifier;
+    }
+    if (!country.isEmpty()) {
+        languages += language + QLatin1Char('_') + country;
+    }
+    languages += language;
+}
+
 static void appendLanguagesFromVariable(QStringList &languages,
                                         const char *envar, bool isList = false)
 {
@@ -102,32 +131,25 @@ static void appendLanguagesFromVariable(QStringList &languages,
     if (!qenvar.isEmpty()) {
         QString value = QFile::decodeName(qenvar);
         if (isList) {
-            languages += value.split(QLatin1Char(':'));
+            foreach(const QString &v, value.split(QLatin1Char(':'), QString::SkipEmptyParts)) {
+                appendLocaleString(languages, v);
+            }
         } else {
-            // Process the value to create possible combinations.
-            QString language, country, modifier, charset;
-            splitLocale(value, language, country, modifier, charset);
-
-            if (!country.isEmpty() && !modifier.isEmpty()) {
-                languages +=   language + QLatin1Char('_')
-                               + country + QLatin1Char('@')
-                               + modifier;
-            }
-            // NOTE: Priority is unclear in case both the country and
-            // the modifier are present. Should really language@modifier be of
-            // higher priority than language_country?
-            // In at least one case (Serbian language), it is better this way.
-            if (!modifier.isEmpty()) {
-                languages += language + QLatin1Char('@') + modifier;
-            }
-            if (!country.isEmpty()) {
-                languages += language + QLatin1Char('_') + country;
-            }
-            languages += language;
+            appendLocaleString(languages, value);
         }
     }
 }
 
+#ifndef Q_OS_UNIX
+static void appendLanguagesFromQLocale(QStringList &languages, const QLocale &locale)
+{
+    const QStringList uiLangs = locale.uiLanguages();
+    Q_FOREACH (QString value, uiLangs) {
+        appendLocaleString(languages, value.replace(QLatin1Char('-'), QLatin1Char('_')));
+    }
+}
+#endif
+
 // Extract the first country code from a list of language_COUNTRY strings.
 // Country code is converted to all lower case letters.
 static QString extractCountry(const QStringList &languages)
@@ -298,7 +320,7 @@ KLocalizedStringPrivateStatics::KLocalizedStringPrivateStatics()
     , scriptModulesToLoad()
 
     , loadTranscriptCalled(false)
-    , ktrs(NULL)
+    , ktrs(nullptr)
 
     , formatters()
 
@@ -333,6 +355,11 @@ void KLocalizedStringPrivateStatics::initializeLocaleLanguages()
     appendLanguagesFromVariable(localeLanguages, "LC_ALL");
     appendLanguagesFromVariable(localeLanguages, "LC_MESSAGES");
     appendLanguagesFromVariable(localeLanguages, "LANG");
+#ifndef Q_OS_UNIX
+    // For non UNIX platforms the environment variables might not
+    // suffice so we add system locale UI languages, too.
+    appendLanguagesFromQLocale(localeLanguages, QLocale::system());
+#endif
 }
 
 KLocalizedString::KLocalizedString()
@@ -614,7 +641,7 @@ QString KLocalizedStringPrivate::toString(const QByteArray &domain,
 
     // Execute any scripted post calls; they cannot modify the final result,
     // but are used to set states.
-    if (s->ktrs != NULL) {
+    if (s->ktrs != nullptr) {
         QStringList pcalls = s->ktrs->postCalls(language);
         foreach (const QString &pcall, pcalls) {
             postTranscript(pcall, language, country, finalTranslation,
@@ -783,7 +810,7 @@ QString KLocalizedStringPrivate::substituteTranscript(const QString &scriptedTra
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP();
 
-    if (s->ktrs == NULL) {
+    if (s->ktrs == nullptr) {
         // Scripting engine not available.
         return QString();
     }
@@ -1045,7 +1072,7 @@ QString KLocalizedStringPrivate::postTranscript(const QString &pcall,
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP();
 
-    if (s->ktrs == NULL) {
+    if (s->ktrs == nullptr) {
         // Scripting engine not available.
         // (Though this cannot happen, we wouldn't be here then.)
         return QString();
@@ -1243,6 +1270,13 @@ QByteArray KLocalizedString::applicationDomain()
     return s->applicationDomain;
 }
 
+QStringList KLocalizedString::languages()
+{
+    KLocalizedStringPrivateStatics *s = staticsKLSP();
+
+    return s->languages;
+}
+
 void KLocalizedString::setLanguages(const QStringList &languages)
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP();
@@ -1358,7 +1392,7 @@ void KLocalizedStringPrivate::loadTranscript()
     QMutexLocker lock(&s->klspMutex);
 
     s->loadTranscriptCalled = true;
-    s->ktrs = NULL; // null indicates that Transcript is not available
+    s->ktrs = nullptr; // null indicates that Transcript is not available
 
     // QPluginLoader is just used to find the plugin
     QPluginLoader loader(QStringLiteral("kf5/ktranscript"));
@@ -1467,18 +1501,18 @@ QString KLocalizedString::translateQt(const char *context,
         if (comment && comment[0]) {
             // Comment given, go for context call.
             KLocalizedStringPrivate::translateRaw(domain, s->languages,
-                                                  comment, sourceText, 0, 0,
+                                                  comment, sourceText, nullptr, 0,
                                                   language, translation);
         } else {
             // Comment not given, go for try-fallback with context.
             if (context && context[0]) {
                 KLocalizedStringPrivate::translateRaw(domain, s->languages,
-                                                      context, sourceText, 0, 0,
+                                                      context, sourceText, nullptr, 0,
                                                       language, translation);
             }
             if (language.isEmpty() || language == s->codeLanguage) {
                 KLocalizedStringPrivate::translateRaw(domain, s->languages,
-                                                      0, sourceText, 0, 0,
+                                                      nullptr, sourceText, nullptr, 0,
                                                       language, translation);
             }
         }
@@ -1520,42 +1554,46 @@ void KLocalizedString::removeQtDomain(const char *domain)
     }
 }
 
+void KLocalizedString::addDomainLocaleDir(const QByteArray &domain, const QString &path) {
+    KCatalog::addDomainLocaleDir(domain, path);
+}
+
 KLocalizedString ki18n(const char *text)
 {
-    return KLocalizedString(NULL, NULL, text, NULL, false);
+    return KLocalizedString(nullptr, nullptr, text, nullptr, false);
 }
 
 KLocalizedString ki18nc(const char *context, const char *text)
 {
-    return KLocalizedString(NULL, context, text, NULL, false);
+    return KLocalizedString(nullptr, context, text, nullptr, false);
 }
 
 KLocalizedString ki18np(const char *singular, const char *plural)
 {
-    return KLocalizedString(NULL, NULL, singular, plural, false);
+    return KLocalizedString(nullptr, nullptr, singular, plural, false);
 }
 
 KLocalizedString ki18ncp(const char *context,
                          const char *singular, const char *plural)
 {
-    return KLocalizedString(NULL, context, singular, plural, false);
+    return KLocalizedString(nullptr, context, singular, plural, false);
 }
 
 KLocalizedString ki18nd(const char *domain, const char *text)
 {
-    return KLocalizedString(domain, NULL, text, NULL, false);
+    return KLocalizedString(domain, nullptr, text, nullptr, false);
 }
 
 KLocalizedString ki18ndc(const char *domain, const char *context,
                          const char *text)
 {
-    return KLocalizedString(domain, context, text, NULL, false);
+    return KLocalizedString(domain, context, text, nullptr, false);
 }
 
 KLocalizedString ki18ndp(const char *domain,
                          const char *singular, const char *plural)
 {
-    return KLocalizedString(domain, NULL, singular, plural, false);
+    return KLocalizedString(domain, nullptr, singular, plural, false);
 }
 
 KLocalizedString ki18ndcp(const char *domain, const char *context,
@@ -1566,40 +1604,40 @@ KLocalizedString ki18ndcp(const char *domain, const char *context,
 
 KLocalizedString kxi18n(const char *text)
 {
-    return KLocalizedString(NULL, NULL, text, NULL, true);
+    return KLocalizedString(nullptr, nullptr, text, nullptr, true);
 }
 
 KLocalizedString kxi18nc(const char *context, const char *text)
 {
-    return KLocalizedString(NULL, context, text, NULL, true);
+    return KLocalizedString(nullptr, context, text, nullptr, true);
 }
 
 KLocalizedString kxi18np(const char *singular, const char *plural)
 {
-    return KLocalizedString(NULL, NULL, singular, plural, true);
+    return KLocalizedString(nullptr, nullptr, singular, plural, true);
 }
 
 KLocalizedString kxi18ncp(const char *context,
                           const char *singular, const char *plural)
 {
-    return KLocalizedString(NULL, context, singular, plural, true);
+    return KLocalizedString(nullptr, context, singular, plural, true);
 }
 
 KLocalizedString kxi18nd(const char *domain, const char *text)
 {
-    return KLocalizedString(domain, NULL, text, NULL, true);
+    return KLocalizedString(domain, nullptr, text, nullptr, true);
 }
 
 KLocalizedString kxi18ndc(const char *domain, const char *context,
                           const char *text)
 {
-    return KLocalizedString(domain, context, text, NULL, true);
+    return KLocalizedString(domain, context, text, nullptr, true);
 }
 
 KLocalizedString kxi18ndp(const char *domain,
                           const char *singular, const char *plural)
 {
-    return KLocalizedString(domain, NULL, singular, plural, true);
+    return KLocalizedString(domain, nullptr, singular, plural, true);
 }
 
 KLocalizedString kxi18ndcp(const char *domain, const char *context,
